// Fifo.cpp: implementation of the CFifo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Fifo.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFifo::CFifo()
{
	data = NULL;
	oldData = 0;
	size = 0;
	oldSize = 0;
	SetSize(0);
}

CFifo::~CFifo()
{
	Clear();
}

UINT CFifo::GetCount(void)
{
	return size;
}

void CFifo::SetSize(UINT newSize)
{
	SaveData();

	data = (BYTE*)malloc(newSize);
	if(oldData != NULL && data != NULL)
	{
		memcpy(data, oldData, oldSize);
	}
	size = newSize;
	ClearOld();
}

void CFifo::SaveData()
{
	ClearOld();
	if(data != NULL)
	{
		oldData = (BYTE*)malloc(size);
		memcpy(oldData, data, size);
		free(data);
		oldSize = size;
		data = NULL;
		size = 0;
	}
}

void	CFifo::DeleteHead(void)
{
	if(size > 0)
	{
		SaveData();
		
		data = (BYTE*)malloc(oldSize-1);
		memcpy(data, oldData+1, oldSize-1);
		size = oldSize-1;
		ClearOld();
	}
}

void CFifo::Clear(void)
{
	if(data != NULL)
	{
		free(data);
		data=NULL;
	}
	size = 0;
	ClearOld();
}

void CFifo::ClearOld(void)
{
	if(oldData != NULL)
	{
		free(oldData);
		oldData=NULL;
	}
	oldSize = 0;
}

BYTE	CFifo::ReadByte(void)
{
	BYTE result=0;
	if(size > 0)
	{
		result = data[0];
		DeleteHead();
	}
	return result;
}

BYTE	CFifo::PeekByte(void)
{
	if(size > 0)
		return data[0];
	else
		return 0;
}

UINT	CFifo::WriteByte(BYTE NewByte)
{
	SetSize(size+1);

	data[size-1] = NewByte;
	return size;
}

UINT CFifo::Peek(BYTE *buffer, UINT MaxLen)
{
	
	int Len;
	if(MaxLen>size)
		Len = size;
	else
		Len = MaxLen;
	if(Len > 0)
		memcpy(buffer, data, Len);
	return Len;
}

UINT CFifo::Read(BYTE *buffer, UINT MaxLen)
{
	int Len;
	if(MaxLen>size)
		Len = size;
	else
		Len = MaxLen;

	if(Len > 0)
	{
		memcpy(buffer, data, Len);
		Delete(Len);
	}
	return Len;
}

UINT CFifo::Write(BYTE* buffer, UINT Len)
{
	UINT	oldS = size;
	SetSize(size+Len);
	memcpy(data+oldS, buffer, Len);
	return size;
}

UINT CFifo::PeekAlphaNumLine(LPSTR *buffer, UINT MaxLen)
{
	int Len;
	if(MaxLen-1>size)
		Len = size;
	else
		Len = MaxLen-1;
	char *b1=new char[Len];
	if(Peek((BYTE*)b1, Len) == 0)
		return 0;
	for(int i=0;i<Len;i++)
	{
		if(isalnum(b1[i]) == 0 && b1[i] != ' ' && b1[i] != '.' && b1[i] != ',')
		{
			memcpy(buffer,b1,i);
			buffer[i] = 0;
			delete b1;
			return i;
		}
	}
	delete b1;
	return -1;
}

void	CFifo::RemoveNonAlpha(void)
{
	while(size>0)
	{
		if( isalpha((char)(PeekByte())) != 0)
			break;
		else
		{
			DeleteHead();
		}
	}
}

void	CFifo::RemoveAlpha(void)
{
	while(size>0)
	{
		if( isalpha((char)(PeekByte())) == 0)
			break;
		else
		{
			DeleteHead();
		}
	}
}

UINT CFifo::Delete(UINT MaxLen)
{
	if(size == 0)
		return 0;
	if(MaxLen>=size)
	{
		Clear();
		return 0;
	}
	
	SaveData();
	size = oldSize-MaxLen;
	data = (BYTE*)malloc(size);
	memcpy(data, oldData+MaxLen, size);
	ClearOld();
	return size;
}