#ifndef _ARRAY_CLASSES_H_INCLUDED_
#define _ARRAY_CLASSES_H_INCLUDED_

#include <fstream.h>

#if !defined BYTE
typedef unsigned char BYTE;
#endif
#if !defined ULONG
typedef unsigned long ULONG;
#endif
//----------------------------
template <class T> class _Array
{
public:
	_Array();
	_Array(_Array*);
	_Array(T *buffer, ULONGLONG count);
	~_Array();

	_Array& Assign(_Array*);

	ULONGLONG	GetSize();
	void	SetSize(ULONGLONG newSize);
	ULONGLONG	GetFullSize();

	ULONGLONG	Add(T newValue);
	ULONGLONG	Insert(ULONGLONG index, T newValue);
	void	Fill(T Value);
	void	Fill(T	Value, ULONGLONG count);
	void	Fill(T	*buffer, ULONGLONG count);
	void	Delete(ULONGLONG index);
	void	Clear();
	T&	Item(ULONGLONG index)
	{
		return data[index];
	};

	T&	operator[](size_t index){return data[index];};
	_Array& operator=(_Array& oldArray)
				{return Assign(&oldArray);}
	friend ostream& operator << (ostream&, const _Array&);
	friend istream& operator >> (istream&, _Array&);

	void	ToStream(ostream&);
	void	FromStream(istream&);
	ULONGLONG	ToBuffer(BYTE* buffer);
	ULONGLONG	FromBuffer(BYTE* buffer);

	T*	GetData();

	T	Min();
	T	Max();
private:
	T		*data;
	ULONGLONG	uiCount;
};

//----------------------------
template <class T> class _PtrArray
{
private:
	ULONGLONG	*data;
	ULONGLONG	uiCount;
	void	SetSize(ULONGLONG);
public:
	_PtrArray();
	_PtrArray(_PtrArray*);
	~_PtrArray();

	ULONGLONG	GetCount();

	ULONGLONG	Add(T*);
	T*		Add();
	ULONGLONG	Insert(ULONGLONG, T*);
	ULONGLONG	Delete(ULONGLONG);
	void	Clear();
	T*	Item(ULONGLONG index);
	T*	operator[](size_t index){return (T*)(data[index]);};
};


//----------------------------
template <class T> class _PtrQueue : public _PtrArray<T>
{
public:
	_PtrQueue();
	~_PtrQueue();

	T*		Peek();
	T*		Pop();
	UINT	Push(T *newItem);
};


//*****************************************************************************
//		_Array template class
//*****************************************************************************
//----------------------------
template <class T> _Array<T>::_Array()
{
	data = new T[0];
	uiCount = 0;
}
//----------------------------
template <class T> _Array<T>::_Array(_Array *source)
{
	uiCount = oldArray->GetSize();
	data = new T[uiCount];
	memcpy(data, oldArray->GetData(), sizeof(T)*uiCount);
}
//----------------------------
template <class T> _Array<T>::_Array(T *buffer, ULONGLONG count)
{
	uiCount = count;
	data = new T[uiCount];
	memcpy(data, buffer, sizeof(T)*uiCount);
}

//----------------------------
template <class T> _Array<T>::~_Array()
{
	delete[] data;
}

//----------------------------
template <class T> _Array<T>& _Array<T>::Assign(_Array<T>* oldArray)
{
	Fill(oldArray->GetData(), oldArray->GetSize());
	return *this;
}

//----------------------------
template <class T> ULONGLONG	_Array<T>::GetSize()
{
	return uiCount;
}

//----------------------------
template <class T> ULONGLONG	_Array<T>::GetFullSize()
{
	return sizeof(ULONGLONG)+sizeof(T)*uiCount;
}

//----------------------------
template <class T> void	_Array<T>::SetSize(ULONGLONG newSize)
{
	T *olddata = data;
	data = new T[newSize];
	if(uiCount > newSize)
		uiCount = newSize;
	memcpy(data, olddata, uiCount*sizeof(T));
	uiCount = newSize;
	delete[] olddata;
}

//----------------------------
template <class T> ULONGLONG	_Array<T>::Add(T newValue)
{
	SetSize(uiCount+1);
	data[uiCount-1] = newValue;
	return uiCount;
}

//----------------------------
template <class T> ULONGLONG	_Array<T>::Insert(ULONGLONG index, T newValue)
{
	SetSize(uiCount+1);

	if(index < uiCount)
	{
		for(ULONGLONG i=(uiCount-1);i>index;i++)
		{
			data[i] = data[i-1];
		}
		data[index] = newValue;
		return index;
	}
	else
	{
		index = uiCount-1;
		data[index] = newValue;
		return index;
	}
}

//----------------------------
template <class T> void	_Array<T>::Fill(T	Value)
{
	for(ULONGLONG i=0;i<uiCount;i++)
		data[i] = Value;
}

//----------------------------
template <class T> void	_Array<T>::Fill(T	Value, ULONGLONG count)
{
	SetSize(count);
	memcpy(data, buffer, sizeof(T)*count);
}

//----------------------------
template <class T> void	_Array<T>::Fill(T	*buffer, ULONGLONG count)
{
	SetSize(count);
	memcpy(data, buffer, sizeof(T)*count);
}

//----------------------------
template <class T> void	_Array<T>::Delete(ULONGLONG index)
{
	if(index < uiCount)
	{
		for(ULONGLONG i=index;i<uiCount-1;i++)
		{
			data[i] = data[i+1];
		}
		SetSize(uiCount-1);
	}
}

//----------------------------
template <class T> void	_Array<T>::Clear()
{
		delete[] data;
		data = new T[0];
		uiCount = 0;
}

//----------------------------
template <class T> T*	_Array<T>::GetData()
{
	return data;
}

//----------------------------
template <class T> ostream& operator << (ostream &out, const _Array<T> &d1)
{
	out.write((char *)&(d1.uiCount), sizeof(ULONGLONG));
	out.write((char*)d1.data, sizeof(T)*d1.uiCount);
    return out;
}

//----------------------------
template <class T> istream& operator >> (istream &in, _Array<T> &d1)
{
	ULONGLONG	newSize;
    in.read((char*)&(newSize), sizeof(ULONGLONG));
	d1.SetSize(newSize);
    in.read((char*)d1.GetData(),newSize*sizeof(T));
	return in;
}

//----------------------------
template <class T> void _Array<T>::ToStream(ostream &out)
{
	out.write((char *)&(uiCount), sizeof(ULONGLONG));
	out.write((char*)data, sizeof(T)*uiCount);
}

//----------------------------
template <class T> void _Array<T>::FromStream(istream &in)
{
	ULONGLONG	newSize;
    in.read((char*)&(newSize), sizeof(ULONGLONG));
	SetSize(newSize);
    in.read((char*)data,newSize*sizeof(T));
}

//----------------------------
template <class T> ULONGLONG _Array<T>::ToBuffer(BYTE* buffer)
{
	ULONGLONG offset=0;
	memcpy(buffer,&(uiCount), sizeof(ULONGLONG));
	offset+=sizeof(ULONGLONG);
	memcpy(buffer+offset,data, sizeof(T)*uiCount);
	return (offset+sizeof(T)*uiCount);
}

//----------------------------
template <class T> ULONGLONG _Array<T>::FromBuffer(BYTE* buffer)
{
	ULONGLONG	newSize,offset=0;
	memcpy(&newSize,buffer,sizeof(ULONGLONG));
	offset+=sizeof(ULONGLONG);
	SetSize(newSize);
	memcpy(data,buffer+offset, sizeof(T)*newSize);
	return (offset+sizeof(T)*newSize);
}

//----------------------------
template <class T> T	_Array<T>::Min()
{
	T result = data[0];
	for(ULONGLONG i=0;i<uiCount;i++)
		if(data[i]<result)
			result = data[i];
	return result;
}
//----------------------------
template <class T> T	_Array<T>::Max()
{
	T result = data[0];
	for(ULONGLONG i=0;i<uiCount;i++)
		if(data[i]>result)
			result = data[i];
	return result;
}

//*****************************************************************************
//		_PtrArray class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
template <class T> void	_PtrArray<T>::SetSize(ULONGLONG newSize)
{
	ULONGLONG *olddata = data;
	data = new ULONGLONG[(size_t)newSize];
	if(uiCount > newSize)
		uiCount = newSize;
	memcpy(data, olddata, ((size_t)uiCount)*sizeof(ULONGLONG));
	uiCount = newSize;
	delete[] olddata;
}
//----------------------------
//		PUBLIC
//----------------------------
template <class T> _PtrArray<T>::_PtrArray()
{
	data = new ULONGLONG[0];
	uiCount = 0;
}
//----------------------------
template <class T> _PtrArray<T>::_PtrArray(_PtrArray<T> *Source)
{
	uiCount = Source->GetCount();
	data = new ULONGLONG[uiCount];
    for(ULONGLONG i=0;i<uiCount;i++)
        data[i] = (ULONGLONG)(Source->Item(i));

}
//----------------------------
template <class T> _PtrArray<T>::~_PtrArray()
{
//	Clear();
	delete[] data;
}
//----------------------------
template <class T> ULONGLONG _PtrArray<T>::GetCount()
{
	return uiCount;
}
//----------------------------
template <class T> ULONGLONG _PtrArray<T>::Add(T *newValue)
{
	SetSize(uiCount+1);
	data[uiCount-1] = (ULONGLONG)newValue;
	return uiCount;
}
//----------------------------
template <class T> T* _PtrArray<T>::Add()
{
	T *newValue = new T();
	Add(newValue);
	return newValue;
}
//----------------------------
template <class T> ULONGLONG _PtrArray<T>::Insert(ULONGLONG index, T *newValue)
{
	SetSize(uiCount+1);
	if(index < uiCount)
	{
		for(ULONGLONG i=(uiCount-1);i>index;i--)
		{
			data[i] = data[i-1];
		}
		data[index] = (ULONGLONG)newValue;
		return index;
	}
	else
	{
		index = uiCount-1;
		data[index] = (ULONGLONG)newValue;
		return index;
	}
}
//----------------------------
template <class T> ULONGLONG	_PtrArray<T>::Delete(ULONGLONG index)
{
	if(index < uiCount)
	{
//		delete (T*)(data[index]);
		for(ULONGLONG i=index;i<uiCount-1;i++)
		{
			data[i] = data[i+1];
		}
		SetSize(uiCount-1);
	}
	return uiCount;
}
//----------------------------
template <class T> T*	_PtrArray<T>::Item(ULONGLONG index)
{
	if(index < uiCount)
		return (T*)(data[index]);
	else
		return NULL;
}
//----------------------------
template <class T> void	_PtrArray<T>::Clear()
{
	try
	{
/*		for(ULONGLONG i=0;i<uiCount;i++)
			delete (T*)(data[i]);
*/
		delete[] data;
		data = new ULONGLONG[0];
		uiCount = 0;
	}
	catch(...)
	{
		delete[] data;
		data = new ULONGLONG[0];
		uiCount = 0;
	}
}

//*****************************************************************************
//		_PtrQueue class
//*****************************************************************************
template <class T> _PtrQueue<T>::_PtrQueue()
	:_PtrArray<T>()
{
}
//----------------------------
template <class T> _PtrQueue<T>::~_PtrQueue()
{
	Clear();
}
//----------------------------
template <class T> T* _PtrQueue<T>::Peek()
{
	if(GetCount() > 0)
		return Item(0);
	else
		return NULL;
}
//----------------------------
template <class T> T* _PtrQueue<T>::Pop()
{
	if(GetCount() > 0)
	{
		T *item = Item(0);
		Delete(0);
		return item;
	}
	else
		return NULL;
}
//----------------------------
template <class T> UINT _PtrQueue<T>::Push(T *newItem)
{
	return (UINT)Add(newItem);
	
}
//----------------------------


#endif