#ifndef _lab_data_h_included_
#define _lab_data_h_included_


#ifdef __BORLANDC__
#include <vcl.h>
#endif

#include "\cpp\arrays.h"

#include <fstream.h>
#if _MSC_VER > 1000
#pragma once
#endif
//*****************************************************************************
//		_DoubleArray class
//*****************************************************************************
class _DoubleArray
{
private:
	double	*data;
	UINT	uiCount;
public:
	_DoubleArray();
	_DoubleArray(_DoubleArray*);
    _DoubleArray(double *buffer, UINT count);
	~_DoubleArray();
	_DoubleArray& Assign(_DoubleArray*);

	UINT	GetSize();
	void	SetSize(UINT newSize);
	UINT	GetFullSize();

	UINT	Add(double newValue);
	UINT	Insert(UINT index, double newValue);
	void	Fill(double Value);
	void	Fill(double	*buffer, UINT count);
	void	Delete(UINT index);
	void	Clear();

	double&	Item(UINT index)
	{
		return data[index];
	};

	double&	operator[](size_t index){return data[index];};
	_DoubleArray& operator=(_DoubleArray& oldArray)
				{return Assign(&oldArray);}
	friend ostream& operator << (ostream&, const _DoubleArray&);
	friend istream& operator >> (istream&, _DoubleArray&);

#ifndef __BORLANDC__
	friend CArchive& operator << (CArchive&, const _DoubleArray&);
	friend CArchive& operator >> (CArchive&, _DoubleArray&);
#endif
	void	ToStream(ostream&);
	void	FromStream(istream&);
	UINT	ToBuffer(BYTE* buffer);
	UINT	FromBuffer(BYTE* buffer);

	double*	GetData();

	double	Min();
	double	Max();

};
//*****************************************************************************
//		_PtrList class
//*****************************************************************************
class _PtrList
{
private:
	ULONG	*data;
	UINT	uiCount;
	void	SetSize(UINT);
public:
	_PtrList();
	_PtrList(_PtrList*);
	~_PtrList();

	UINT	GetCount();

	UINT	Add(void*);
	UINT	Insert(UINT, void*);
	UINT	Delete(UINT);
	void	Clear();
	void*	Item(UINT index);
	void*	operator[](size_t index){return (void*)(data[index]);};
};
//*****************************************************************************
//		_SrtingList class
//*****************************************************************************
#ifndef __BORLANDC__
class _StringList : public _PtrList
{
public:
	UINT	Add(CString newString);
	UINT	Add(char *newString);
	UINT	Insert(UINT index, CString newString);
	UINT	Delete(UINT);
	void	Clear();
	CString	Item(UINT index);
//	CString	operator[](size_t index){return *Item(index);};
public:
	_StringList();
	~_StringList();

	int		Find(CString);
};
#endif
//*****************************************************************************
//		_IntArray class
//*****************************************************************************
class _IntArray
{
private:
	int	*data;
	UINT	uiCount;
public:
	_IntArray();
	_IntArray(_IntArray*);
	~_IntArray();
	_IntArray& Assign(_IntArray*);

	UINT	GetSize();
	void	SetSize(UINT newSize);

	UINT	Add(int newValue);
	UINT	Insert(UINT index, int newValue);
	void	Fill(int Value);
	void	Fill(int	*buffer, UINT count);
	void	Delete(UINT index);
	void	Clear();

	int&	Item(UINT index)
	{
		return data[index];
	};

	int&	operator[](size_t index){return data[index];};
	_IntArray& operator=(_IntArray& oldArray)
				{return Assign(&oldArray);}
	friend ostream& operator << (ostream&, const _IntArray&);
	friend istream& operator >> (istream&, _IntArray&);
	void	ToStream(ostream&);
	void	FromStream(istream&);

	int*	GetData();

	int		Min();
	int		Max();

};
//*****************************************************************************
//		_LabData class
//*****************************************************************************
class _LabData
{
private:
	_PtrList	data;
public:
	_LabData();
	~_LabData();

	void	Assign(_LabData* labData);

	UINT	GetFullSize();

	UINT	GetDimCount();
	void	SetDims(UINT newCount);
	UINT	AddDim();											//	return dim count
	UINT	AddDim(_DoubleArray*);								//	return dim count
	_DoubleArray*	AddDimR();
	UINT	DeleteDim(UINT);									//	return dim count
	void	SetNewDimSize(UINT dim, UINT newSize);
	double	Item(UINT dim, UINT index);
	_DoubleArray*	Dim(UINT dim);
	_DoubleArray&	operator[](size_t index){return *(_DoubleArray*)(data[index]);};

#ifndef __BORLANDC__
	friend CArchive& operator << (CArchive&, _LabData*&);
	friend CArchive& operator >> (CArchive&, _LabData&);
#endif
	void	ToStream(ostream&);
	void	FromStream(istream&);
	UINT	ToBuffer(BYTE* buffer);
	UINT	FromBuffer(BYTE* buffer);
};
//*****************************************************************************
//		_LabGraphData class
//*****************************************************************************
class _LabGraphData : public _LabData
{
private:
	char	*Name;
	void	SetIniValues();
public:
	_LabGraphData();
	_LabGraphData(_LabData*);
	_LabGraphData(_LabGraphData*);
	~_LabGraphData();

	void	Assign(_LabGraphData*);
	char*	GetName();
	void	SetName(char *newName);
	UINT	GetFullSize();
	
	UINT	LineColor;
	SHORT	LineWidth;
	SHORT	LineStyle;

	UINT	PointColor;
	SHORT	PointStyle;

	BOOL	Fill;
	UINT	FillColor;

	BOOL	LineToBase;
	UINT	LineToBaseColor;
	double	Base;

#ifndef __BORLANDC__
	friend CArchive& operator << (CArchive&, _LabData*&);
	friend CArchive& operator >> (CArchive&, _LabData&);
#endif
	void	ToStream(ostream&);
	void	FromStream(istream&);

	UINT	ToBuffer(BYTE* buffer);
	UINT	FromBuffer(BYTE* buffer);
};
//*****************************************************************************
//		_LabGraphDataArray class
//*****************************************************************************
class _LabGraphDataArray
{
private:
	_PtrList	data;
public:
	_LabGraphDataArray();
	~_LabGraphDataArray();
	void	Clear();

	UINT	GetCount();
	UINT	GetFullSize();

	_LabGraphData*	Add();
	UINT			Add(_LabGraphData*);
	UINT			Delete(UINT);
	_LabGraphData*	Item(UINT);
	UINT			Find(char*);
	_LabGraphData&	operator[](size_t index){return *(_LabGraphData*)(data[index]);};

#ifndef __BORLANDC__
	friend CArchive& operator << (CArchive&, _LabGraphDataArray&);
	friend CArchive& operator >> (CArchive&, _LabGraphDataArray&);
#endif
	void	ToStream(ostream&);
	void	FromStream(istream&);

	void	ToFile(LPSTR FileName);
	BOOL	FromFile(LPSTR FileName);
};
//*****************************************************************************

//*****************************************************************************
//*****************************************************************************
#ifndef __BORLANDC__
class CLabGraphDataArray : public _PtrArray<_LabGraphData>
{
public:
	CLabGraphDataArray();
	~CLabGraphDataArray();

	UINT			Find(char*);

	void	ToStream(ostream&);
	void	FromStream(istream&);
	void	ToFile(LPSTR FileName);
	BOOL	FromFile(LPSTR FileName);
};



#endif
//*****************************************************************************

#endif