#include "stdafx.h"
#include "labdata.h"
//*****************************************************************************
//	implementation
//*****************************************************************************
//		_DoubleArray class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
//----------------------------
//		PUBLIC
//----------------------------
_DoubleArray::_DoubleArray()
{
	data = new double[0];
	uiCount = 0;
}
//----------------------------
_DoubleArray::_DoubleArray(_DoubleArray *oldArray)
{
	uiCount = oldArray->GetSize();
	data = new double[uiCount];
	memcpy(data, oldArray->GetData(), sizeof(double)*uiCount);
}
//----------------------------
_DoubleArray::_DoubleArray(double *buffer, UINT count)
{
	uiCount = count;
	data = new double[uiCount];
	memcpy(data, buffer, sizeof(double)*uiCount);
}
//----------------------------
_DoubleArray& _DoubleArray::Assign(_DoubleArray* oldArray)
{
	Fill(oldArray->GetData(), oldArray->GetSize());
	return *this;
}
//----------------------------
_DoubleArray::~_DoubleArray()
{
	delete data;
}
//----------------------------
UINT	_DoubleArray::GetSize()
{
	return uiCount;
}
//----------------------------
UINT	_DoubleArray::GetFullSize()
{
	return sizeof(UINT)+sizeof(double)*uiCount;
}
//----------------------------
void	_DoubleArray::SetSize(UINT newSize)
{
	double *olddata = data;
	data = new double[newSize];
	if(uiCount > newSize)
		uiCount = newSize;
	memcpy(data, olddata, uiCount*sizeof(double));
	uiCount = newSize;
	delete olddata;
}
//----------------------------
UINT	_DoubleArray::Add(double newValue)
{
	SetSize(uiCount+1);
	data[uiCount-1] = newValue;
	return uiCount;
}
//----------------------------
UINT	_DoubleArray::Insert(UINT index, double newValue)
{
	SetSize(uiCount+1);

	if(index < uiCount)
	{
		for(UINT i=(uiCount-1);i>index;i++)
		{
			data[i] = data[i-1];
		}
		data[index] = newValue;
		return index;
	}
	else
	{
		index = uiCount-1;
		data[index] = newValue;
		return index;
	}
}
//----------------------------
void	_DoubleArray::Fill(double	Value)
{
	for(UINT i=0;i<uiCount;i++)
		data[i] = Value;
}
//----------------------------
void	_DoubleArray::Fill(double	*buffer, UINT count)
{
	SetSize(count);
	memcpy(data, buffer, sizeof(double)*count);
}
//----------------------------
void	_DoubleArray::Delete(UINT index)
{
	if(index < uiCount)
	{
		for(UINT i=index;i<uiCount-1;i++)
		{
			data[i] = data[i+1];
		}
		SetSize(uiCount-1);
	}
}
//----------------------------
void	_DoubleArray::Clear()
{
	delete data;
	data = new double[0];
	uiCount = 0;
}
//----------------------------
double*	_DoubleArray::GetData()
{
	return data;
}
//---------------------------------------------------------------------------
ostream& operator << (ostream &out, const _DoubleArray &d1)
{
	out.write((char *)&(d1.uiCount), sizeof(UINT));
	out.write((char*)d1.data, sizeof(double)*d1.uiCount);
    return out;
}
//---------------------------------------------------------------------------
istream& operator >> (istream &in, _DoubleArray &d1)
{
	UINT	newSize;
    in.read((char*)&(newSize), sizeof(UINT));
	d1.SetSize(newSize);
    in.read((char*)d1.GetData(),newSize*sizeof(double));
	return in;
}
//---------------------------------------------------------------------------
CArchive& operator << (CArchive &out, const _DoubleArray &d1)
{
	out.Write((char *)&(d1.uiCount), sizeof(UINT));
	out.Write((char*)d1.data, sizeof(double)*d1.uiCount);
    return out;
}
//---------------------------------------------------------------------------
CArchive& operator >> (CArchive &in, _DoubleArray &d1)
{
	UINT	newSize;
    in.Read((char*)&(newSize), sizeof(UINT));
	d1.SetSize(newSize);
    in.Read((char*)d1.GetData(),newSize*sizeof(double));
	return in;
}
//---------------------------------------------------------------------------
void _DoubleArray::ToStream(ostream &out)
{
	out.write((char *)&(uiCount), sizeof(UINT));
	out.write((char*)data, sizeof(double)*uiCount);
}
//---------------------------------------------------------------------------
void _DoubleArray::FromStream(istream &in)
{
	UINT	newSize;
    in.read((char*)&(newSize), sizeof(UINT));
	SetSize(newSize);
    in.read((char*)data,newSize*sizeof(double));
}
//---------------------------------------------------------------------------
UINT _DoubleArray::ToBuffer(BYTE* buffer)
{
	UINT offset=0;
	memcpy(buffer,&(uiCount), sizeof(UINT));
	offset+=sizeof(UINT);
	memcpy(buffer+offset,data, sizeof(double)*uiCount);
	return (offset+sizeof(double)*uiCount);
}
//---------------------------------------------------------------------------
UINT _DoubleArray::FromBuffer(BYTE* buffer)
{
	UINT	newSize,offset=0;
	memcpy(&newSize,buffer,sizeof(UINT));
	offset+=sizeof(UINT);
	SetSize(newSize);
	memcpy(data,buffer+offset, sizeof(double)*newSize);
	return (offset+sizeof(double)*newSize);
}
//----------------------------
double	_DoubleArray::Min()
{
	double result = data[0];
	for(UINT i=0;i<uiCount;i++)
		if(data[i]<result)
			result = data[i];
	return result;
}
//----------------------------
double	_DoubleArray::Max()
{
	double result = data[0];
	for(UINT i=0;i<uiCount;i++)
		if(data[i]>result)
			result = data[i];
	return result;
}
//*****************************************************************************
//		_IntArray class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
//----------------------------
//		PUBLIC
//----------------------------
_IntArray::_IntArray()
{
	data = new int[0];
	uiCount = 0;
}
//----------------------------
_IntArray::_IntArray(_IntArray *oldArray)
{
	uiCount = oldArray->GetSize();
	data = new int[uiCount];
	memcpy(data, oldArray->GetData(), sizeof(int)*uiCount);
}
//----------------------------
_IntArray& _IntArray::Assign(_IntArray* oldArray)
{
	Fill(oldArray->GetData(), oldArray->GetSize());
	return *this;
}
//----------------------------
_IntArray::~_IntArray()
{
	delete data;
}
//----------------------------
UINT	_IntArray::GetSize()
{
	return uiCount;
}
//----------------------------
void	_IntArray::SetSize(UINT newSize)
{
	int *olddata = data;
	data = new int[newSize];
	if(uiCount > newSize)
		uiCount = newSize;
	memcpy(data, olddata, uiCount*sizeof(int));
	uiCount = newSize;
	delete olddata;
}
//----------------------------
UINT	_IntArray::Add(int newValue)
{
	SetSize(uiCount+1);
	data[uiCount-1] = newValue;
	return uiCount;
}
//----------------------------
UINT	_IntArray::Insert(UINT index, int newValue)
{
	SetSize(uiCount+1);

	if(index < uiCount)
	{
		for(UINT i=(uiCount-1);i>index;i++)
		{
			data[i] = data[i-1];
		}
		data[index] = newValue;
		return index;
	}
	else
	{
		index = uiCount-1;
		data[index] = newValue;
		return index;
	}
}
//----------------------------
void	_IntArray::Fill(int	Value)
{
	for(UINT i=0;i<uiCount;i++)
		data[i] = Value;
}
//----------------------------
void	_IntArray::Fill(int	*buffer, UINT count)
{
	SetSize(count);
	memcpy(data, buffer, sizeof(int)*count);
}
//----------------------------
void	_IntArray::Delete(UINT index)
{
	if(index < uiCount)
	{
		for(UINT i=index;i<uiCount-1;i++)
		{
			data[i] = data[i+1];
		}
		SetSize(uiCount-1);
	}
}
//----------------------------
void	_IntArray::Clear()
{
	delete data;
	data = new int[0];
	uiCount = 0;
}
//----------------------------
int*	_IntArray::GetData()
{
	return data;
}
//---------------------------------------------------------------------------
ostream& operator << (ostream &out, const _IntArray &d1)
{
	out.write((char *)&(d1.uiCount), sizeof(UINT));
	out.write((char*)d1.data, sizeof(int)*d1.uiCount);
    return out;
}
//---------------------------------------------------------------------------
istream& operator >> (istream &in, _IntArray &d1)
{
	UINT	newSize;
    in.read((char*)&(newSize), sizeof(UINT));
	d1.SetSize(newSize);
    in.read((char*)d1.GetData(),newSize*sizeof(int));
	return in;
}
//---------------------------------------------------------------------------
void _IntArray::ToStream(ostream &out)
{
	out.write((char *)&(uiCount), sizeof(UINT));
	out.write((char*)data, sizeof(int)*uiCount);
}
//---------------------------------------------------------------------------
void _IntArray::FromStream(istream &in)
{
	UINT	newSize;
    in.read((char*)&(newSize), sizeof(UINT));
	SetSize(newSize);
    in.read((char*)data,newSize*sizeof(int));
}
//----------------------------
int	_IntArray::Min()
{
	int result = data[0];
	for(UINT i=0;i<uiCount;i++)
		if(data[i]<result)
			result = data[i];
	return result;
}
//----------------------------
int	_IntArray::Max()
{
	int result = data[0];
	for(UINT i=0;i<uiCount;i++)
		if(data[i]>result)
			result = data[i];
	return result;
}
//*****************************************************************************
//		_PtrList class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
void	_PtrList::SetSize(UINT newSize)
{
	ULONG *olddata = data;
	data = new ULONG[newSize];
	if(uiCount > newSize)
		uiCount = newSize;
	memcpy(data, olddata, uiCount*sizeof(UINT));
	uiCount = newSize;
	delete olddata;
}
//----------------------------
//		PUBLIC
//----------------------------
_PtrList::_PtrList()
{
	data = new ULONG[0];
	uiCount = 0;
}
//----------------------------
_PtrList::_PtrList(_PtrList *Source)
{
	uiCount = Source->GetCount();
	data = new ULONG[uiCount];
    for(UINT i=0;i<uiCount;i++)
        data[i] = (ULONG)(Source->Item(i));

}
//----------------------------
_PtrList::~_PtrList()
{
	delete[] data;
}
//----------------------------
UINT _PtrList::GetCount()
{
	return uiCount;
}
//----------------------------
UINT _PtrList::Add(void *newValue)
{
	SetSize(uiCount+1);
	data[uiCount-1] = (ULONG)newValue;
	return uiCount;
}
//----------------------------
UINT _PtrList::Insert(UINT index, void *newValue)
{
	SetSize(uiCount+1);
	if(index < uiCount)
	{
		for(UINT i=(uiCount-1);i>index;i--)
		{
			data[i] = data[i-1];
		}
		data[index] = (ULONG)newValue;
		return index;
	}
	else
	{
		index = uiCount-1;
		data[index] = (ULONG)newValue;
		return index;
	}
}
//----------------------------
UINT	_PtrList::Delete(UINT index)
{
	if(index < uiCount)
	{
		for(UINT i=index;i<uiCount-1;i++)
		{
			data[i] = data[i+1];
		}
		SetSize(uiCount-1);
	}
	return uiCount;
}
//----------------------------
void*	_PtrList::Item(UINT index)
{
	if(index < uiCount)
		return (void*)data[index];
	else
		return NULL;
}
//----------------------------
void	_PtrList::Clear()
{
	delete[] data;
	data = new ULONG[0];
	uiCount = 0;
}
//*****************************************************************************
//		_StringList class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
//----------------------------
//		PROTECTED
//----------------------------
UINT	_StringList::Add(CString newString)
{
	CString *str = new CString();
	*str = newString;
	return _PtrList::Add(str);
}
//----------------------------
UINT	_StringList::Add(char *newString)
{
	return Add(CString(newString));
}
//----------------------------
UINT	_StringList::Insert(UINT index, CString newString)
{
	CString *str = new CString();
	*str = newString;
	return _PtrList::Insert(index, str);
}
//----------------------------
CString	_StringList::Item(UINT index)
{
	return *((CString*)(_PtrList::Item(index)));
}
//----------------------------
UINT	_StringList::Delete(UINT index)
{
	if(index < GetCount())
	{
		delete (CString*)(_PtrList::Item(index));
		return _PtrList::Delete(index);
	}

	return GetCount();
}
//----------------------------
void	_StringList::Clear()
{
	for(UINT i=0;i<GetCount();i++)
		delete (CString*)(_PtrList::Item(i));

	_PtrList::Clear();
}
//----------------------------
//		PUBLIC
//----------------------------
_StringList::_StringList()
	:_PtrList()
{
}
//----------------------------
_StringList::~_StringList()
{
	Clear();
}
//----------------------------
int	_StringList::Find(CString string)
{
	for(UINT i=0;i<GetCount();i++)
	{
		if(string == Item(i))
			return i;
	}

	return -1;
}
//*****************************************************************************
//		_LabData class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
//----------------------------
//		PUBLIC
//----------------------------

_LabData::_LabData()
{
}
//----------------------------
_LabData::~_LabData()
{
	while(DeleteDim(0) > 0);
}
//---------------------------------------------------------------------------
void	_LabData::Assign(_LabData* labData)
{
	UINT	dims = labData->GetDimCount();
	for(UINT i=0;i<dims;i++)
	{
		_DoubleArray *newDim = AddDimR();
		newDim->Assign(labData->Dim(i));
	}
}
//----------------------------
UINT	_LabData::GetFullSize()
{
	UINT	fullSize=0;
	for(UINT i=0;i<data.GetCount();i++)
		fullSize+=Dim(i)->GetFullSize();
	return fullSize;
}
//----------------------------
UINT	_LabData::GetDimCount()
{
	return data.GetCount();
}
//----------------------------
void	_LabData::SetDims(UINT newSize)
{
	if(newSize > data.GetCount())
	{
		while(newSize > AddDim());
	}
	else
	{
		while(newSize < data.GetCount())
			DeleteDim(data.GetCount()-1);
	}
}
//----------------------------
UINT	_LabData::AddDim()
{
	data.Add(new _DoubleArray());
	return data.GetCount();
}
//----------------------------
UINT	_LabData::AddDim(_DoubleArray *newDim)
{
	data.Add(newDim);
	return data.GetCount();
}
//----------------------------
_DoubleArray*	_LabData::AddDimR()
{
	return Dim(AddDim()-1);
}
//----------------------------
UINT	_LabData::DeleteDim(UINT index)
{
	try
	{
		if(data.GetCount() == 0)
			return 0;
		delete (_DoubleArray*)(data[index]);
		data.Delete(index);
		return data.GetCount();
	}
	catch(...)
	{
		return -1;
	}
}
//----------------------------
void	_LabData::SetNewDimSize(UINT dim, UINT newSize)
{
	((_DoubleArray*)(data[dim]))->SetSize(newSize);
}
//----------------------------
double	_LabData::Item(UINT dim, UINT index)
{
	return ((_DoubleArray*)(data[dim]))->Item(index);
}
//----------------------------
_DoubleArray*	_LabData::Dim(UINT dim)
{
	return (_DoubleArray*)(data[dim]);
}
//---------------------------------------------------------------------------
void _LabData::ToStream(ostream &out)
{
	UINT	Count = data.GetCount();
	out.write((char *)&(Count), sizeof(UINT));
	for(UINT i=0;i<Count;i++)
		Dim(i)->ToStream(out);
}
//---------------------------------------------------------------------------
void _LabData::FromStream(istream &in)
{
	UINT	newCount;
    in.read((char*)&(newCount), sizeof(UINT));
	for(UINT i=0;i<newCount;i++)
	{
		_DoubleArray *newDim = new _DoubleArray();
		newDim->FromStream(in);
		data.Add(newDim);
	}
}
//---------------------------------------------------------------------------
CArchive& operator >> (CArchive &in, _LabData &Data)
{
	UINT	newCount;
    in.Read((char*)&(newCount), sizeof(UINT));
	for(UINT i=0;i<newCount;i++)
	{
		_DoubleArray *newDim = new _DoubleArray();
		in >> *newDim;
		Data.AddDim(newDim);
	}
	return in;
}
//---------------------------------------------------------------------------
CArchive& operator << (CArchive &out, _LabData &Data)
{
	_LabData *pData = &Data;
	UINT	Count = pData->GetDimCount();
	out.Write((char *)&(Count), sizeof(UINT));
	for(UINT i=0;i<Count;i++)
		out << *(pData->Dim(i));
	return out;
}
//---------------------------------------------------------------------------
UINT _LabData::ToBuffer(BYTE* buffer)
{
	UINT	offset=0;
	UINT	Count = data.GetCount();
	memcpy(buffer,&(Count), sizeof(UINT));
	offset+=sizeof(UINT);
	for(UINT i=0;i<Count;i++)
	{
		offset+=Dim(i)->ToBuffer(buffer+offset);
	}
	return offset;
}
//---------------------------------------------------------------------------
UINT _LabData::FromBuffer(BYTE* buffer)
{
	UINT	newCount,offset=0;
    memcpy(&(newCount),buffer, sizeof(UINT));
	offset+=sizeof(UINT);
	for(UINT i=0;i<newCount;i++)
	{
		_DoubleArray *newDim = new _DoubleArray();
		offset+=newDim->FromBuffer(buffer+offset);
		data.Add(newDim);
	}
	return offset;
}
//*****************************************************************************
//		_LabGraphData class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
void _LabGraphData::SetIniValues()
{
	Name=new char[0];

	LineColor = 0xFFFFFF;	//	white
	LineWidth = 1;
	LineStyle = 1;			//	solid

	PointColor = 0xFFFFFF;	//	white
	PointStyle = 0;			//	none

	Fill = FALSE;
	FillColor = 0xFFFFFF;	//	white

	LineToBase = FALSE;
	LineToBaseColor = 0xFFFFFF;	//	white
	Base = 0.0;
}
//----------------------------
//		PUBLIC
//----------------------------
//---------------------------------------------------------------------------
_LabGraphData::_LabGraphData()
	:_LabData()
{
	SetIniValues();
}
//---------------------------------------------------------------------------
_LabGraphData::_LabGraphData(_LabData* labData)
{
	_LabData::Assign(labData);
	SetIniValues();
}
//---------------------------------------------------------------------------
_LabGraphData::_LabGraphData(_LabGraphData* labData)
{
	Assign(labData);
}
//---------------------------------------------------------------------------
_LabGraphData::~_LabGraphData()
{
	if(Name != NULL)
	{
		delete Name;
		Name = NULL;
	}
}
void _LabGraphData::Assign(_LabGraphData *labData)
{
	_LabData::Assign((_LabData*)labData);
	SetIniValues();
	LineColor = labData->LineColor;
	LineWidth = labData->LineWidth;
	LineStyle = labData->LineStyle;

	PointColor = labData->PointColor;
	PointStyle = labData->PointStyle;

	Fill = labData->Fill;
	FillColor = labData->FillColor;

	LineToBase = labData->LineToBase;
	LineToBaseColor = labData->LineToBaseColor;
	Base = labData->Base;

	SetName(labData->GetName());
}
//----------------------------
UINT	_LabGraphData::GetFullSize()
{
	UINT	fullSize=_LabData::GetFullSize();
	
	UINT Len = (UINT)strlen(Name);
	fullSize+=sizeof(char)*Len;//Name len
	fullSize+=sizeof(UINT);//LineColor
	fullSize+=sizeof(SHORT);//LineWidth
	fullSize+=sizeof(SHORT);//LineStyle
	fullSize+=sizeof(UINT);//PointColor
	fullSize+=sizeof(SHORT);//PointStyle
	fullSize+=sizeof(BOOL);//Fill
	fullSize+=sizeof(UINT);//FillColor
	fullSize+=sizeof(BOOL);//LineToBase
	fullSize+=sizeof(UINT);//LineToBaseColor
	fullSize+=sizeof(double);//Base
	
	return fullSize;
}
//---------------------------------------------------------------------------
char* _LabGraphData::GetName()
{
	return Name;
}
//---------------------------------------------------------------------------
void _LabGraphData::SetName(char *newName)
{
	delete Name;
	size_t len = strlen(newName);
	Name = new char[len+1];
	memcpy(Name, newName,len);
	Name[len] = 0;
}
//---------------------------------------------------------------------------
CArchive& operator >> (CArchive &in, _LabGraphData &Data)
{
	in >> *((_LabData*)&Data);
	UINT Len;
	in.Read((char *)&Len, sizeof(UINT));
	char *newName = new char[Len+1];
	in.Read((char *)newName, sizeof(char)*Len);
	newName[Len] = 0;
	Data.SetName(newName);
	delete newName;

	in.Read((char *)&(Data.LineColor), sizeof(UINT));
	in.Read((char *)&(Data.LineWidth), sizeof(SHORT));
	in.Read((char *)&(Data.LineStyle), sizeof(SHORT));

	in.Read((char *)&(Data.PointColor), sizeof(UINT));
	in.Read((char *)&(Data.PointStyle), sizeof(SHORT));

	in.Read((char *)&(Data.Fill), sizeof(BOOL));
	in.Read((char *)&(Data.FillColor), sizeof(UINT));
	
	in.Read((char *)&(Data.LineToBase), sizeof(BOOL));
	in.Read((char *)&(Data.LineToBaseColor), sizeof(UINT));
	in.Read((char *)&(Data.Base), sizeof(double));
	return in;
}
//---------------------------------------------------------------------------
CArchive& operator << (CArchive &out, _LabGraphData &Data)
{
	out << *((_LabData*)&Data);
	UINT Len = (UINT)strlen(Data.GetName());
	out.Write((char *)&Len, sizeof(UINT));
	out.Write(Data.GetName(), sizeof(char)*Len);

	out.Write((char *)&(Data.LineColor), sizeof(UINT));
	out.Write((char *)&(Data.LineWidth), sizeof(SHORT));
	out.Write((char *)&(Data.LineStyle), sizeof(SHORT));

	out.Write((char *)&(Data.PointColor), sizeof(UINT));
	out.Write((char *)&(Data.PointStyle), sizeof(SHORT));

	out.Write((char *)&(Data.Fill), sizeof(BOOL));
	out.Write((char *)&(Data.FillColor), sizeof(UINT));
	
	out.Write((char *)&(Data.LineToBase), sizeof(BOOL));
	out.Write((char *)&(Data.LineToBaseColor), sizeof(UINT));
	out.Write((char *)&(Data.Base), sizeof(double));
	return out;
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void _LabGraphData::ToStream(ostream &out)
{
	_LabData::ToStream(out);
	UINT Len = (UINT)strlen(Name);
	out.write((char *)&Len, sizeof(UINT));
	out.write((char *)Name, sizeof(char)*Len);

	out.write((char *)&LineColor, sizeof(UINT));
	out.write((char *)&LineWidth, sizeof(SHORT));
	out.write((char *)&LineStyle, sizeof(SHORT));

	out.write((char *)&PointColor, sizeof(UINT));
	out.write((char *)&PointStyle, sizeof(SHORT));

	out.write((char *)&Fill, sizeof(BOOL));
	out.write((char *)&FillColor, sizeof(UINT));
	
	out.write((char *)&LineToBase, sizeof(BOOL));
	out.write((char *)&LineToBaseColor, sizeof(UINT));
	out.write((char *)&Base, sizeof(double));
}
//---------------------------------------------------------------------------
void _LabGraphData::FromStream(istream &in)
{
	_LabData::FromStream(in);
	UINT Len;
	in.read((char *)&Len, sizeof(UINT));
	delete Name;
	Name = new char[Len+1];
	in.read((char *)Name, sizeof(char)*Len);
	Name[Len] = 0;

	in.read((char *)&LineColor, sizeof(UINT));
	in.read((char *)&LineWidth, sizeof(SHORT));
	in.read((char *)&LineStyle, sizeof(SHORT));

	in.read((char *)&PointColor, sizeof(UINT));
	in.read((char *)&PointStyle, sizeof(SHORT));

	in.read((char *)&Fill, sizeof(BOOL));
	in.read((char *)&FillColor, sizeof(UINT));
	
	in.read((char *)&LineToBase, sizeof(BOOL));
	in.read((char *)&LineToBaseColor, sizeof(UINT));
	in.read((char *)&Base, sizeof(double));
}
//---------------------------------------------------------------------------
UINT _LabGraphData::ToBuffer(BYTE* buffer)
{
	int offset=0;
	offset += _LabData::ToBuffer(buffer);
	UINT Len = (UINT)strlen(Name);
	memcpy(buffer+offset,&Len, sizeof(UINT));
	offset+=sizeof(UINT);

	memcpy(buffer+offset,Name, sizeof(char)*Len);
	offset+=sizeof(char)*Len;

	memcpy(buffer+offset,&LineColor, sizeof(UINT));
	offset+=sizeof(UINT);

	memcpy(buffer+offset,&LineWidth, sizeof(SHORT));
	offset+=sizeof(SHORT);

	memcpy(buffer+offset,&LineStyle, sizeof(SHORT));
	offset+=sizeof(SHORT);

	memcpy(buffer+offset,&PointColor, sizeof(UINT));
	offset+=sizeof(UINT);

	memcpy(buffer+offset,&PointStyle, sizeof(SHORT));
	offset+=sizeof(SHORT);

	memcpy(buffer+offset,&Fill, sizeof(BOOL));
	offset+=sizeof(BOOL);

	memcpy(buffer+offset,&FillColor, sizeof(UINT));
	offset+=sizeof(UINT);
	
	memcpy(buffer+offset,&LineToBase, sizeof(BOOL));
	offset+=sizeof(BOOL);

	memcpy(buffer+offset,&LineToBaseColor, sizeof(UINT));
	offset+=sizeof(UINT);

	memcpy(buffer+offset,&Base, sizeof(double));
	offset+=sizeof(double);
	return offset;
}
//---------------------------------------------------------------------------
UINT _LabGraphData::FromBuffer(BYTE* buffer)
{
	int offset = _LabData::FromBuffer(buffer);
	UINT Len;
	memcpy(&Len, buffer+offset, sizeof(UINT));
	offset+=sizeof(UINT);
	delete Name;
	Name = new char[Len+1];
	memcpy(Name, buffer+offset, sizeof(char)*Len);
	offset+=sizeof(char)*Len;
	Name[Len] = 0;

	memcpy(&LineColor, buffer+offset, sizeof(UINT));
	offset+=sizeof(UINT);
	memcpy(&LineWidth, buffer+offset, sizeof(SHORT));
	offset+=sizeof(SHORT);
	memcpy(&LineStyle, buffer+offset, sizeof(SHORT));
	offset+=sizeof(SHORT);

	memcpy(&PointColor, buffer+offset, sizeof(UINT));
	offset+=sizeof(UINT);
	memcpy(&PointStyle, buffer+offset, sizeof(SHORT));
	offset+=sizeof(SHORT);

	memcpy(&Fill, buffer+offset, sizeof(BOOL));
	offset+=sizeof(BOOL);
	memcpy(&FillColor, buffer+offset, sizeof(UINT));
	offset+=sizeof(UINT);
	
	memcpy(&LineToBase, buffer+offset, sizeof(BOOL));
	offset+=sizeof(BOOL);
	memcpy(&LineToBaseColor, buffer+offset, sizeof(UINT));
	offset+=sizeof(UINT);
	memcpy(&Base, buffer+offset, sizeof(double));
	offset+=sizeof(double);
	return offset;
}
//*****************************************************************************
//		_LabGraphDataArray class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
//----------------------------
//		PUBLIC
//----------------------------
_LabGraphDataArray::_LabGraphDataArray()
{
}
//----------------------------
_LabGraphDataArray::~_LabGraphDataArray()
{
	Clear();
}
//----------------------------
void _LabGraphDataArray::Clear()
{
	while(data.GetCount() > 0)
		Delete(0);
}
//----------------------------
UINT _LabGraphDataArray::GetCount()
{
	return data.GetCount();
}
//----------------------------
UINT	_LabGraphDataArray::GetFullSize()
{
	UINT	fullSize=0;
	for(UINT i=0;i<data.GetCount();i++)
		fullSize+=Item(i)->GetFullSize();
	return fullSize;
}
//----------------------------
_LabGraphData*	_LabGraphDataArray::Add()
{
	_LabGraphData *newData = new _LabGraphData();
	data.Add(newData);
	return newData;
}
//----------------------------
UINT _LabGraphDataArray::Add(_LabGraphData *newData)
{
	data.Add(newData);
	return data.GetCount();
}
//----------------------------
UINT _LabGraphDataArray::Delete(UINT index)
{
	if(index == -1)
	{
		Clear();
	}
	else
	{
		if(index < data.GetCount())
		{
			delete (_LabGraphData*)(data.Item(index));
			data.Delete(index);
		}
	}
	return data.GetCount();
}
//----------------------------
_LabGraphData*	_LabGraphDataArray::Item(UINT index)
{
	if(index < data.GetCount())
		return (_LabGraphData*)(data.Item(index));
	else
		return NULL;
}
UINT _LabGraphDataArray::Find(char *name)
{
	CString Name = CString(name);
	for(UINT i=0;i<GetCount();i++)
		if( CString(Item(i)->GetName()) == Name)
			return i;

	return -1;
}
//----------------------------
CArchive& operator << (CArchive &out, _LabGraphDataArray &Data)
{
	UINT	Count = Data.GetCount();
	out.Write((char *)&(Count), sizeof(UINT));
	for(UINT i=0;i<Count;i++)
		out << *(Data.Item(i));
	return out;
}
//----------------------------
CArchive& operator >> (CArchive &in, _LabGraphDataArray &Data)
{
	UINT	newCount;
    in.Read((char*)&(newCount), sizeof(UINT));
	for(UINT i=0;i<newCount;i++)
	{
		_LabGraphData *newData = new _LabGraphData();
		in >> *newData;
		Data.Add(newData);
	}
	return in;
}
//----------------------------
void _LabGraphDataArray::ToStream(ostream &out)
{
	UINT	Count = data.GetCount();
	out.write((char *)&(Count), sizeof(UINT));
	for(UINT i=0;i<Count;i++)
		Item(i)->ToStream(out);
}
//----------------------------
void _LabGraphDataArray::FromStream(istream &in)
{
	UINT	newCount;
    in.read((char*)&(newCount), sizeof(UINT));
	for(UINT i=0;i<newCount;i++)
	{
		_LabGraphData *newData = new _LabGraphData();
		newData->FromStream(in);
		data.Add(newData);
	}
}
//----------------------------
void _LabGraphDataArray::ToFile(LPSTR FileName)
{
	ofstream ofs(FileName, ios::out|ios::binary);
    ofs.write("LabGraphDataArray_1.0",21);
	ToStream(ofs);
}
//----------------------------
BOOL _LabGraphDataArray::FromFile(LPSTR FileName)
{
	ifstream ifs(FileName, ios::in|ios::binary);
	char string[22];
	ifs.read(string, 21);
	string[21] = 0;
	if(strcmp(string,"LabGraphDataArray_1.0") != 0)
		return FALSE;
	else
	{
		FromStream(ifs);
		return TRUE;
	}
}
//---------------------------------------------------------------------------



//*****************************************************************************
//		CLabGraphDataArray class
//*****************************************************************************
//----------------------------
//		PRIVATE
//----------------------------
//----------------------------
//		PUBLIC
//----------------------------
CLabGraphDataArray::CLabGraphDataArray()
	:_PtrArray<_LabGraphData>()
{
}
//----------------------------
CLabGraphDataArray::~CLabGraphDataArray()
{
	Clear();
}
//----------------------------
UINT CLabGraphDataArray::Find(char *name)
{
	CString Name = CString(name);
	for(UINT i=0;i<GetCount();i++)
		if( CString(Item(i)->GetName()) == Name)
			return i;

	return -1;
}
//----------------------------
void CLabGraphDataArray::ToStream(ostream &out)
{
	UINT	Count = (UINT)GetCount();
	out.write((char *)&(Count), sizeof(UINT));
	for(UINT i=0;i<Count;i++)
		Item(i)->ToStream(out);
}
//----------------------------
void CLabGraphDataArray::FromStream(istream &in)
{
	UINT	newCount;
    in.read((char*)&(newCount), sizeof(UINT));
	for(UINT i=0;i<newCount;i++)
	{
		_LabGraphData *newData = new _LabGraphData();
		newData->FromStream(in);
		Add(newData);
	}
}
//----------------------------
void CLabGraphDataArray::ToFile(LPSTR FileName)
{
	ofstream ofs(FileName, ios::out|ios::binary);
    ofs.write("LabGraphDataArray_1.0",21);
	ToStream(ofs);
}
//----------------------------
BOOL CLabGraphDataArray::FromFile(LPSTR FileName)
{
	ifstream ifs(FileName, ios::in|ios::binary);
	char string[22];
	ifs.read(string, 21);
	string[21] = 0;
	if(strcmp(string,"LabGraphDataArray_1.0") != 0)
		return FALSE;
	else
	{
		FromStream(ifs);
		return TRUE;
	}
}
//----------------------------
