#include "stdafx.h"
#include "exports.h"

#include "socketCallbacks.h"
#include "Sockets.h"
#include "tcpclientdefs.h"

CClientSockets	*ClientSockets=NULL;
UINT	LastSocketId=-1;

bool	GetMessageStruct(CFifo* Fifo, TCPDataStruct &Data);
long	GetControlSum(TCPDataStruct &Data);



extern "C" __declspec(dllexport) UINT CreateSocket(UINT	ProcessFunction)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	if(ClientSockets == NULL)
	{
		ClientSockets = new CClientSockets();
	}
	if(ClientSockets == NULL)
		return -1;
	CClientSocket *newSocket = ClientSockets->New();

	if(newSocket != NULL)
	{
		if(!newSocket->Create(0,SOCK_STREAM,NULL))
		{
//			char string[255];
//			sprintf(string, "Create Error code = %d", newSocket->GetLastError());
//			MessageBox(NULL, string, "Debug", MB_OK);
			delete newSocket;
			return -1;
		}
		newSocket->SocketOnClose = OnSocketClose;
		newSocket->SocketOnConnect = OnSocketConnect;
		newSocket->SocketOnReceive = OnSocketReceive;
		newSocket->SocketOnSend = OnSocketSend;
		newSocket->SocketOnError = OnSocketError;
		newSocket->ProcessFunction = (TCPFUNCTION1)ProcessFunction;
		return newSocket->SocketId;
	}
	else
		return -1;
}

extern "C" __declspec(dllexport) BOOL DestroySocket(UINT SocketId)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	if(ClientSockets != NULL)
	{
		CClientSocket *Socket = ClientSockets->FindId(SocketId);
		if(Socket == NULL)
			return FALSE;
		if(Socket->ProcessFunction != NULL)
			Socket->ProcessFunction = NULL;
		
		return ClientSockets->DeleteId(SocketId);
	}
	else
		return FALSE;
}

extern "C" __declspec(dllexport) BOOL Connect(UINT SocketId, LPCTSTR Address, UINT Port)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	CClientSocket *Socket = ClientSockets->FindId(SocketId);
	if(Socket != NULL)
	{
		Socket->AsyncSelect(FD_READ | FD_WRITE | FD_OOB | FD_ACCEPT | FD_CONNECT | FD_CLOSE );
		if(Socket->Connect(Address, Port) != 0)
			return TRUE;
		else
		{
			char string[255];
			sprintf(string, "Error code = %d\nAddress = %s, Port = %d ", Socket->GetLastError(), Address, Port);
			MessageBox(NULL, string, "Error", MB_OK);
			return FALSE;
		}
	
	}
	else
		return FALSE;
}

extern "C" __declspec(dllexport) BOOL Send(UINT SocketId, BYTE *buffer, UINT count)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	CClientSocket *Socket = ClientSockets->FindId(SocketId);
	if(Socket == NULL)
		return FALSE;
	Socket->OutBuffer.Write(buffer, (int)count);
	OnSocketSend(Socket);
	return TRUE;
}


extern "C" __declspec(dllexport) UINT GetReceiveLen(UINT SocketId)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	CClientSocket *Socket = ClientSockets->FindId(SocketId);
	if(Socket == NULL)
		return 0;

	return Socket->InBuffer.GetCount();
}

extern "C" __declspec(dllexport) BOOL Receive_Buffer(UINT SocketId, BYTE *buffer, UINT MaxLen)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	CClientSocket *Socket = ClientSockets->FindId(SocketId);
	if(Socket == NULL)
		return FALSE;

	int size = Socket->InBuffer.GetCount();
	if(size > (int)MaxLen)
		size = (int)MaxLen;
	if(size > 0)
	{
		BYTE *newBuffer = new BYTE[size];
		int newSize = Socket->InBuffer.Peek(newBuffer, size);
		if(newSize != -1 && newSize == size)
		{
			Socket->InBuffer.Delete(newSize);
			memcpy(buffer, newBuffer, size);
			delete newBuffer;
			return TRUE;
		}
		delete newBuffer;
	}

	return FALSE;
}

extern "C" __declspec(dllexport) BOOL Receive_Struct(UINT SocketId, TCPDataStruct *Data)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	CClientSocket *Socket = ClientSockets->FindId(SocketId);
	if(Socket == NULL)
		return FALSE;

	if(GetMessageStruct(&(Socket->InBuffer), *Data))
	{
		return TRUE;
	}
	return FALSE;
}

//********************************
bool	GetMessageStruct(CFifo* Fifo, TCPDataStruct &Data)
{
	if(Fifo->GetCount() < _TCP_HEADER_SIZE_)
		return false;
	BYTE *buffer = new BYTE[_TCP_HEADER_SIZE_];
	UINT Len = _TCP_HEADER_SIZE_;

	if(Fifo->Peek(buffer, Len) != Len)
	{
		delete buffer;
		return false;
	}

	memcpy(&(Data.funcId),buffer,sizeof(int));
	int offset = sizeof(int);

	memcpy(&(Data.dataLen),buffer+offset,sizeof(int));
	offset+=sizeof(int);

	memcpy(&(Data.controlSumm),buffer+offset,sizeof(long));
	offset += sizeof(long);

	UINT MaxLen = _TCP_HEADER_SIZE_+Data.dataLen;
	if(Fifo->GetCount() < MaxLen)
	{
		delete buffer;
		return false;
	}
	delete buffer;
	buffer = new BYTE[MaxLen];
	Fifo->Delete(_TCP_HEADER_SIZE_);
	if(Fifo->Read(Data.data,Data.dataLen) != (UINT)(Data.dataLen))
	{
		delete buffer;
		return false;
	}
	delete buffer;

	if(Data.controlSumm == GetControlSum(Data))
		return true;
	else
	{
		return false;
	}
}
//---------------------------------------------------------------------------
/*long	GetControlSum(TCPDataStruct &Data)
{
	long controlSumm = 0;
	for(UINT i=0;i<Data.dataLen;i++)
		controlSumm+=(long)(Data.data[i]);
	return controlSumm;
}
*/
//********************************